/*	
 *	@filename apimage.js
 *	@description Javascript of APImage mod for SMF 1.1.x and 2.x.x
 *
 *	@package APImage
 *	@version 0.1.8
 *	@copyright 2011 AntePrimorac
 *	@license MPL 2.0
 *
 *	This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 *	If a copy of the MPL was not distributed with this file,
 *	You can obtain one at http://mozilla.org/MPL/2.0/.
 *	
 *	@contributor Ante Primorac
 *	http://anteprimorac.from.hr
 *
 *	The orginal code is http://custom.simplemachines.org/mods/index.php?mod=3388
 *
*/

(function($) {
	$.fn.APImage = function() {
		function load_options() {
			if(typeof APImage_options == "object") {
				APImage_options["thumb_max_width"] = parseInt(APImage_options["thumb_max_width"]);
				APImage_options["thumb_max_height"] = parseInt(APImage_options["thumb_max_height"]);
				APImage_options["gallery_thumb_max_width"] = parseInt(APImage_options["gallery_thumb_max_width"]);
				APImage_options["thumb_max_height"] = parseInt(APImage_options["gallery_thumb_max_height"]);
				
				if(APImage_options["enabled"] != "true") APImage_options["enabled"] = false;
				if((APImage_options["thumb_max_width"]+"").length == 0 || APImage_options["thumb_max_width"] == 0) APImage_options["thumb_max_width"] = false;
				if((APImage_options["thumb_max_height"]+"").length == 0 || APImage_options["thumb_max_height"] == 0) APImage_options["thumb_max_height"] = false;
				if((APImage_options["gallery_thumb_max_width"]+"").length == 0 || APImage_options["gallery_thumb_max_width"] == 0) APImage_options["gallery_thumb_max_width"] = false;
				if((APImage_options["gallery_thumb_max_height"]+"").length == 0 || APImage_options["gallery_thumb_max_height"] == 0) APImage_options["gallery_thumb_max_height"] = false;
			}
			else var APImage_options = false;
		};
		load_options();
		
		function vpD() {
			var w, h, l, t;
			if(self.innerHeight) w = self.innerWidth, h = self.innerHeight;
			else if(document.documentElement && document.documentElement.clientHeight) w = document.documentElement.clientWidth, h = document.documentElement.clientHeight;
			else if(document.body) w = document.body.clientWidth, w = document.body.clientHeight;
			if(self.pageYOffset) l = self.pageXOffset, t = self.pageYOffset;
			else if(document.documentElement && document.documentElement.scrollTop) l = document.documentElement.scrollLeft, t = document.documentElement.scrollTop;
			else if(document.body) l = document.body.scrollLeft, t = document.body.scrollTop;
			return {"w" : w, "h" : h, "l" : l, "t" : t};
		};
		
		function load_image(e) {
			var
				e_src = $(e).attr("data-image"),
				e_width = parseInt($(e).attr("data-width")),
				e_height = parseInt($(e).attr("data-height"));
			if($(e).hasClass("loaded")) {
				if($(e).hasClass("ap-image-vo")) {
					$(".ap-image-viewer").removeClass("loading");
					$(".ap-image-viewer .image").html($("<img src=\""+e_src+"\" />"));
					image_size();
				}
			}
			else {
				$(e).addClass("loading");
				$(e).css({
					"width": APImage_options["thumb_max_width"] == false ? "100%" : ((e_width != 0 && e_width < APImage_options["thumb_max_width"]) ? e_width : (APImage_options["thumb_max_width"] - 20)) + "px",
					"height": APImage_options["thumb_max_height"] == false ? "auto" : ((e_height != 0 && e_height < APImage_options["thumb_max_height"]) ? e_height : (APImage_options["thumb_max_height"] - 20)) + "px"
				});
				var img = new Image();
				img.src = e_src;
				$(img).bind("load", function() {
					var width = (e_width != 0 && e_width < APImage_options["thumb_max_width"]) ? e_width : (img.naturalWidth > APImage_options["thumb_max_width"] - 20 ? APImage_options["thumb_max_width"]  - 20: img.naturalWidth);
					var height = (e_height != 0 && e_height < APImage_options["thumb_max_height"]) ? e_height : (img.naturalHeight > APImage_options["thumb_max_height"] - 20 ? APImage_options["thumb_max_height"] - 20 : img.naturalHeight);
					$(e).removeClass("loading").addClass("loaded");
					$(e).html("<img src=\""+e_src+"\" width=\""+width+"\" height=\""+height+"\" />");
					$(e).css({
						"width": width + "px",
						"height": height + "px"
					});
					if($(e).hasClass("ap-image-vo")) {
						$(".ap-image-viewer").removeClass("loading");
						$(".ap-image-viewer .image").html($("<img src=\""+e_src+"\" />"));
						image_size();
					}
				});
			}
		};
		
		function image_size() {
			if($("body").has(".ap-image-viewer").length) {
				if($(".ap-image-viewer").hasClass("loading")) return false;
				var width = $(window).width() < 1100 ? $(window).width() - 100 : 1000, height = $(window).height() < 750 ? $(window).height() - 100 : 650;
				
				var img = $(".ap-image-viewer .image img"), img2 = new Image();
				img2.src = $(img).attr("src");
				
				width = img2.naturalWidth > width ? width : img2.naturalWidth;
				height = img2.naturalHeight > height ? height : img2.naturalHeight;
				
				$(img).attr("width", width).attr("height", height);
				
				var top = bottom = ($(window).height() - height) / 2,
					right = left = ($(window).width() - width) / 2;
				
				$(".ap-image-viewer").css({
					"width" : width+"px",
					"height" : height+"px",
					"top" : top+"px",
					"right" : right+"px",
					"bottom" : bottom+"px",
					"left" : left+"px"
				});
			}
		};
		
		
		function main(b) {
			var e = $(b).find(".ap-image"), g = $(b).find(".ap-image-gallery");
			if(APImage_options["enabled"] == "true") {

				// Standalone images
				if(e.length) {
					e.each(function() {
						var el = $(this);
						if(el.hasClass("ap-image") && !el.hasClass("loading") && !el.hasClass("loaded") && !el.closest(".ap-image-gallery").length) {
							load_image(el);
							
							$(el).live("click", function() {
								$(this).addClass("ap-image-vo");
								var
									ap_viewer_bg = $("<div class=\"ap-image-viewer-bg\"></div>"),
									ap_viewer = $("<div class=\"ap-image-viewer"+($(this).hasClass("loading") ? " loading" : "")+"\" />"),
									self = this;
								ap_viewer.append($("<div class=\"actions\"><div class=\"fullscreen\"></div><div class=\"close\"></div></div><div class=\"image\"></div>"));
								$("body").append(ap_viewer_bg);
								$("body").append(ap_viewer);
								//$(".ap-image-viewer, .ap-image-viewer-bg").fadeIn("slow");
								if($(this).hasClass("loaded")) {
									$(".ap-image-viewer .image").html($("<img src=\""+$(this).attr("data-image")+"\" />"));
									image_size();
								}
								else load_image($(this));
		
								$(".ap-image-viewer-bg").live("click", function() {
									/*$(".ap-image-viewer, .ap-image-viewer-bg").fadeOut("slow", function() {
										$(this).remove();
									});*/
									$(".ap-image-viewer, .ap-image-viewer-bg").remove();
									$(self).removeClass("ap-image-vo");
								});
							});
						}
					});
				}
				
				// Gallery
				if(g.length && APImage_options["enabled"] == "true") {
					g.each(function() {
						if($(this).hasClass("ap-image-gallery")) {
							var gallery = $(this), els = $(this).find(".ap-image");
							if(els.length) {
								var b = 0;
								gallery.find(".images").css({
									"max-width" : gallery.parent().width() + "px",
									"width" : gallery.parent().width() + "px",
									"height" : (parseInt(APImage_options["gallery_thumb_max_height"]) + 15) + "px"
								});
								gallery.find(".viewer").live("click", function() {
									$(this).addClass("ap-image-vo");
									var
										ap_viewer_bg = $("<div class=\"ap-image-viewer-bg\"></div>"),
										ap_viewer = $("<div class=\"ap-image-viewer"+($(this).hasClass("loading") ? " loading" : "")+"\" />"),
										self = this;
									ap_viewer.append($("<div class=\"actions\"><div class=\"fullscreen\"></div><div class=\"close\"></div></div><div class=\"image\"></div>"));
									$("body").append(ap_viewer_bg);
									$("body").append(ap_viewer);
									//$(".ap-image-viewer, .ap-image-viewer-bg").fadeIn("slow");
									$(".ap-image-viewer .image").html($("<img src=\""+$(this).find("img").attr("src")+"\" />"));
									image_size();
			
									$(".ap-image-viewer-bg").live("click", function() {
										/*$(".ap-image-viewer, .ap-image-viewer-bg").fadeOut("slow", function() {
											$(this).remove();
										});*/
										$(".ap-image-viewer, .ap-image-viewer-bg").remove();
										$(self).removeClass("ap-image-vo");
									});
								});
								els.each(function() {
									var el = $(this);
									if(el.hasClass("ap-image") && !el.hasClass("loading") && !el.hasClass("loaded")) {
										var
											el_src = $(el).attr("data-image"),
											el_width = parseInt($(el).attr("data-width")),
											el_height = parseInt($(el).attr("data-height"));
										if($(el).hasClass("loaded")) {
											if($(el).hasClass("ap-image-vo")) {
												$(".ap-image-viewer").removeClass("loading");
												$(".ap-image-viewer .image").html($("<img src=\""+el_src+"\" />"));
												image_size();
											}
										}
										else {
											if(b == 0) $(el).addClass("current");
											b++;
											$(el).addClass("loading");
											$(el).css({
												"width": APImage_options["gallery_thumb_max_width"] == false ? "100%" : ((el_width != 0 && el_width < APImage_options["gallery_thumb_max_width"]) ? el_width : (APImage_options["gallery_thumb_max_width"] - 20)) + "px",
												"height": APImage_options["gallery_thumb_max_height"] == false ? "auto" : ((el_height != 0 && el_height < APImage_options["gallery_thumb_max_height"]) ? el_height : (APImage_options["gallery_thumb_max_height"] - 20)) + "px"
											});
											var img = new Image();
											img.src = el_src;
											$(img).bind("load", function() {
												$(el).removeClass("loading").addClass("loaded");
												$(el).html("<img src=\""+el_src+"\" width=\""+((el_width != 0 && el_width < APImage_options["gallery_thumb_max_width"]) ? el_width : (APImage_options["gallery_thumb_max_width"] - 20))+"\" height=\""+((el_height != 0 && el_height < APImage_options["gallery_thumb_max_height"]) ? el_height : (APImage_options["gallery_thumb_max_height"] - 20))+"\" />");
												if($(el).hasClass("ap-image-vo")) {
													$(".ap-image-viewer").removeClass("loading");
													$(".ap-image-viewer .image").html($("<img src=\""+el_src+"\" />"));
													image_size();
												}
												if($(el).hasClass("current")) {
													gallery.find(".viewer").html($("<img src=\""+el_src+"\" width=\""+(gallery.parent().width() - 10)+"\" />"));
												}
											});
										}
										
										$(el).live("click", function() {
											var vp_d = vpD(), fw = gallery.find(".viewer").height();
											gallery.find(".images .list .current").removeClass("current");
											$(this).addClass("current");
											gallery.find(".viewer").html($("<img src=\""+$(this).attr("data-image")+"\" width=\""+(gallery.parent().width() - 10)+"\" />"));
											var lw = gallery.find(".viewer").height();
											window.scrollTo(vp_d.l, vp_d.t + (lw - fw));
										});
									}
								});
							}
							else $(this).html("U ovoj galeriji nema slika!");
						}
					});
				}
			
				$(".ap-image-viewer .actions .close").live("click", function() {
					$(".ap-image-viewer, .ap-image-viewer-bg").remove();
				});
				$(".ap-image-viewer .actions .fullscreen").live("click", function() {
					if($(this).hasClass("off")) {
						image_size();
						$(this).removeClass("off");
					}
					else {
						var img = new Image();
						img.src = $(".ap-image-viewer").find(".image img").attr("src");
						var width = img.naturalWidth > $(window).width() ? $(window).width() : img.naturalWidth;
						var height = img.naturalHeight > $(window).height() ? $(window).height() : img.naturalHeight;
						
						$(".ap-image-viewer").find(".image img").attr("width", width).attr("height", height);
						$(".ap-image-viewer").css({
							"top" : (($(window).height() - height) / 2) + "px",
							"right" : (($(window).width() - width) / 2) + "px",
							"bottom" : (($(window).height() - height) / 2) + "px",
							"left" : (($(window).width() - width) / 2) + "px"
						});
						$(this).addClass("off");
					}
				});
			}
			else {
				if(e.length) {
					$(e).each(function() {
						if($(this).closest(".ap-image-gallery").length) {
							$(this).closest(".ap-image-gallery").before($("<img src=\""+$(this).attr("data-image")+"\" width=\""+$(this).attr("data-width")+"\" height=\""+$(this).attr("data-height")+"\" />"));
							$(this).remove();
						}
						else {
							$(this).before($("<img src=\""+$(this).attr("data-image")+"\" width=\""+$(this).attr("data-width")+"\" height=\""+$(this).attr("data-height")+"\" />"));
							$(this).remove();
						}
					});
					$(".ap-image-gallery").remove();
				}
			}
		};
		
		return main($(this));
	};
	
	$(document).ready(function() {
		$("body").APImage();
		$('input[type="submit"]').live("click", function() {
			$("body").not(".ap-image, .ap-image-gallery, .ap-image-viewer, .ap-image-viewer-bg").bind("DOMNodeInserted", function() {
				$("body").APImage();
			});
		});
	});
})(jQuery);